//
// Created by great.zheng on 2022/3/18.
//
#define LOG_TAG "uartJNI"
#include <jni.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <termios.h>
#include <string.h>
#include <unistd.h>
#include <stdio.h>
#include <malloc.h>

#define SPEED_DEFAULT B115200

static jint fd;
static unsigned int baudspeed;
static struct termios options;

char* jstringtochar( JNIEnv *env, jstring jstr )
{
    char* rtn = NULL;
    jclass clsstring = (*env)->FindClass(env,"java/lang/String");
    jstring strencode = (*env)->NewStringUTF(env,"utf-8");
    jmethodID mid = (*env)->GetMethodID(env,clsstring, "getBytes", "(Ljava/lang/String;)[B");
    jbyteArray barr= (jbyteArray)(*env)->CallObjectMethod(env,jstr, mid, strencode);
    jsize alen = (*env)->GetArrayLength(env,barr);
    jbyte* ba = (*env)->GetByteArrayElements(env,barr, JNI_FALSE);
    if (alen > 0)
    {
        rtn = (char*)malloc(alen + 1);
        memcpy(rtn, ba, alen);
        rtn[alen] = 0;
    }
    (*env)->ReleaseByteArrayElements(env,barr, ba, 0);
    return rtn;
}

JNIEXPORT jint JNICALL Java_com_quectel_uartapp_Uart_uartOpen(JNIEnv *env, jobject cls, jstring path)
{
    //const jbyte *str;

    char* str = jstringtochar(env,path);
    if (str == NULL) {
        printf("native uart GetStringUTFChars failed");
        return -1;
    }

    fd = open(str, O_RDWR | O_NOCTTY | O_NDELAY);
    if (fd < 0) {
        printf("native uart open %s failed", str);
        return -1;
    }

    if (tcflush(fd, TCIOFLUSH) < 0) {
        printf("native uart flush uart port error");
        return -1;
    }

    if (tcgetattr(fd, &options) < 0) {
        printf("native uart get port settings failed");
        return -1;
    }

    options.c_cc[VTIME]    = 0;
    options.c_cc[VMIN]     = 1;
    options.c_cflag &= ~CSIZE;
    options.c_cflag |= (CS8 | CLOCAL | CREAD);
    options.c_iflag = IGNPAR;
    options.c_oflag = 0;
    options.c_lflag = 0;
    cfsetospeed(&options, SPEED_DEFAULT);
    cfsetispeed(&options, SPEED_DEFAULT);

    if (tcsetattr(fd, TCSANOW, &options) < 0) {
        printf("native uart set port error");
        return -1;
    }

    fcntl(fd, F_SETFL, 0);
    (*env)->ReleaseStringUTFChars(env, path, str);

    return 0;
}

JNIEXPORT void JNICALL Java_com_quectel_uartapp_Uart_uartClose(JNIEnv *env, jobject cls)
{
    printf("native uart_close");
    close(fd);
}

JNIEXPORT jint JNICALL Java_com_quectel_uartapp_Uart_uartCtrl(JNIEnv *env, jobject cls, jint baudrate)
{
    if(baudrate == 9600)
        baudspeed = B9600;
    else if(baudrate == 19200)
        baudspeed = B19200;
    else if(baudrate == 115200)
        baudspeed = B115200;
    else if(baudrate == 921600)
        baudspeed = B921600;
    else
        baudspeed = B115200;

    cfsetospeed(&options, baudspeed);
    cfsetispeed(&options, baudspeed);
    if (tcsetattr(fd, TCSANOW, &options) < 0) {
        printf("native uart set port error");
        return -1;
    }

    return 0;
}

JNIEXPORT jint JNICALL Java_com_quectel_uartapp_Uart_uartRead(JNIEnv *env, jobject cls, jobject buffer)
{
    int i, size;
    jbyte buf[1024];
    uint8_t *ptr_buf;

    memset(buf, 0, 1024);
    size = read(fd, buf, 1024);
    if (size < 0) {
        printf("native uart read error");
        return -1;
    }

    ptr_buf = (uint8_t *)(*env)->GetDirectBufferAddress(env, buffer);
    if (!ptr_buf) {
        printf("native uart GetDirectBufferAddress error");
        return -1;
    }
    for (i = 0; i < size; i++)
        memcpy(ptr_buf, buf, size);

    return size;
}

JNIEXPORT jint JNICALL Java_com_quectel_uartapp_Uart_uartWrite(JNIEnv *env, jobject cls, jstring buf)
{
int size, len;
const jbyte *str;

str = (*env)->GetStringUTFChars(env, buf, NULL);
if (str == NULL) {
    printf("native uart_write GetStringUTFChars failed");
return -1;
}

len = strlen(str);
size = write(fd, str, len);
if(size != len) {
    printf("native uart_write failed");
return -1;
}
(*env)->ReleaseStringUTFChars(env, buf, str);

return size;
}

//static const JNINativeMethod methods[] = {
//        {"uartOpen",  "(Ljava/lang/String;)I", (void *)uart_open },
//        {"uartClose", "()V",                   (void *)uart_close},
//        {"uartCtrl",  "(I)I",                  (void *)uart_ctrl },
//        {"uartRead",  "(Ljava/lang/Object;)I", (void *)uart_read },
//        {"uartWrite", "(Ljava/lang/String;)I", (void *)uart_write},
//};

//extern "C" JNIEXPORT jint JNICALL
//JNI_OnLoad(JavaVM *jvm, void *reserved)
//{
//    JNIEnv *env;
//    jclass cls;
//
//    if ((*jvm)->GetEnv(jvm, (void **)&env, JNI_VERSION_1_4)) {
//        return JNI_ERR;
//    }
//    cls = (*env)->FindClass(env, "com/hardctrl/Uart");
//    if (cls == NULL) {
//        return JNI_ERR;
//    }
//
//    if ((*env)->RegisterNatives(env, cls, methods, sizeof(methods) / sizeof(methods[0])) < 0)
//        return JNI_ERR;
//
//    return JNI_VERSION_1_4;
//}


