package com.quectel.uartapp;
/**
 * Date:2022/3/18
 * Author:Great Zheng
 * Company:Quectel
 * Desc:Android Application demo for UART
 */

import androidx.appcompat.app.AppCompatActivity;

import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.os.Message;
import android.text.method.ScrollingMovementMethod;
import android.util.Log;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.CompoundButton;
import android.widget.EditText;
import android.widget.Spinner;
import android.widget.TextView;

import java.io.File;
import java.nio.ByteBuffer;
import java.nio.CharBuffer;
import java.nio.charset.Charset;
import java.nio.charset.CharsetDecoder;
import java.util.ArrayList;
import java.util.List;

public class MainActivity extends AppCompatActivity {

    // Used to load the 'native-lib' library on application startup.
    static {
        System.loadLibrary("uart-lib");
    }

    private Handler mHhandler;
    private Looper mLooper;
    private Spinner comPort;
    private TextView dataRX;
    private Spinner rate;
    private boolean openFlag = false;
    private Button sendChar;
    private Button ReceiveData;
    private Uart ttyHSLx;
    private String ReadDataString;
    private EditText txBuf;
    private String txString;
    private CheckBox OpenCheck;
    List<String>uart_ports = new ArrayList<>();
    private readThread rdThread;

    private final int dataLength = 1024;
    private final int msgTypeInfo = 0;
    private final int msgTypeData = 1;
    private static final String LOG_TAG = "UartDemo";

    class readThread extends Thread {
        @Override
        public void run() {
            super.run();
            ByteBuffer byteBuffer = ByteBuffer.allocateDirect(dataLength);
            int readLen;
            while (true) {
                byteBuffer.clear();
                readLen = ttyHSLx.uartRead(byteBuffer);
                byteBuffer.limit(readLen);
                if (readLen < 0) {
                    Log.e(LOG_TAG, "read failed");
                } else {
                    Log.e(LOG_TAG,"read "+readLen);
                    try {
                        CharBuffer charBuffer;
                        Charset charset = Charset.forName("UTF-8");
                        CharsetDecoder decoder = charset.newDecoder();
                        charBuffer = decoder.decode(byteBuffer);
                        Log.d(LOG_TAG,"readLen"+readLen+", charBufferLen:"+charBuffer.length());
                        ReadDataString = charBuffer.toString();
                    } catch (Exception ex) {
                        ex.printStackTrace();
                    }
                    if (ReadDataString!=null && ReadDataString.length() > 0) {
                        Message newMsg = new Message();
                        newMsg.what = msgTypeData;
                        mHhandler.sendMessage(newMsg);
                    }
                }
            }
        }
    }
    private void readUartNode(){
        String[] list ={"/dev/ttyMSM0","/dev/ttyMSM1","/dev/ttyMSM2","/dev/ttyHS0","/dev/ttyHS1","/dev/ttyHS2",
        "/dev/ttyHSL0","/dev/ttyHSL1","/dev/ttyHSL2"};
        File node;
        for(String str:list){
            node = new File(str);
            if(node.exists()){
                uart_ports.add(str);
            }
        }
    }
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        comPort = (Spinner) findViewById(R.id.spinnerDeviceNode);
        rate = (Spinner) findViewById(R.id.spinnerBaudRate);

        readUartNode();

        ArrayAdapter<String> adapterPort = new ArrayAdapter(MainActivity.this,android.R.layout.simple_spinner_item,uart_ports);
        adapterPort.setDropDownViewResource(android.R.layout.simple_list_item_checked);
        comPort.setAdapter(adapterPort);
        comPort.setSelection(0);

        ArrayAdapter<CharSequence> adapterRate = ArrayAdapter.createFromResource(this, R.array.Baudrate,
                android.R.layout.simple_spinner_item);
        adapterRate.setDropDownViewResource(android.R.layout.simple_list_item_checked);
        rate.setAdapter(adapterRate);
        rate.setSelection(2);

        ttyHSLx = new Uart();
        txBuf = (EditText) findViewById(R.id.editTextTextTX);
        dataRX = (TextView) findViewById(R.id.textViewtextRX);
        sendChar = (Button) findViewById(R.id.buttonSendText);
        ReceiveData = (Button) findViewById(R.id.buttonEmptyRXText);
        sendChar.setOnClickListener(new View.OnClickListener() {
            public void onClick(View v) {
                if (!openFlag) {
                    Log.e(LOG_TAG, "Please open device first");
                    return;
                }

                txString = txBuf.getText().toString();
                int rt = ttyHSLx.uartWrite(txString);
                if (rt != txString.length()) {
                    Log.e(LOG_TAG, "send failed");
                } else {
                    Log.e(LOG_TAG, "send succeeded");
                }
            }
        });

        ReceiveData.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                dataRX.setText("");
            }
        });

        OpenCheck = (CheckBox) findViewById(R.id.checkBoxOpenPort);
        assert OpenCheck != null;
        OpenCheck.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {
            @Override
            public void onCheckedChanged(CompoundButton compoundButton, boolean b) {
                if (b) {
                    Spinner portName = (Spinner) findViewById(R.id.spinnerDeviceNode);
                    Spinner portRate = (Spinner) findViewById(R.id.spinnerBaudRate);
                    assert portName != null;
                    String portNode = (String)portName.getSelectedItem();
                    int rt = ttyHSLx.uartOpen(portNode);
                    if (rt != 0) {
                        openFlag = false;
                        Log.e(LOG_TAG, "open port failed");
                        return;
                    }
                    assert portRate != null;
                    rt = ttyHSLx.uartCtrl(Integer.parseInt((String) portRate.getSelectedItem()));
                    if (rt != 0) {
                        openFlag = false;
                        Log.e(LOG_TAG, "set baudrate failed");
                        return;
                    }
                    openFlag = true;
                    Log.e(LOG_TAG, "open port succeeded");
                    rdThread = new readThread();
                    rdThread.start();
                } else {
                    openFlag = false;
                    rdThread.interrupt();
                    ttyHSLx.uartClose();
                }
            }
        });
        comPort.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
            @Override
            public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
                OpenCheck.setChecked(false);
            }

            @Override
            public void onNothingSelected(AdapterView<?> parent) {

            }
        });
        rate.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
            @Override
            public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
                OpenCheck.setChecked(false);
            }

            @Override
            public void onNothingSelected(AdapterView<?> parent) {

            }
        });
        mHhandler = new Handler(Looper.getMainLooper(), new Handler.Callback() {
            @Override
            public boolean handleMessage(Message message) {
                if (message.what == msgTypeInfo) {} // Empty information
                else {                           // Received data from UART
                    String textSend = dataRX.getText().toString();
                    textSend += ReadDataString;
                    dataRX.setText(textSend);
                    dataRX.setMovementMethod(new ScrollingMovementMethod()); // Scoll Bar
                }
                return false;
            }
        });
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        ttyHSLx.uartClose();
    }

    /**
     * A native method that is implemented by the 'native-lib' native library,
     * which is packaged with this application.
     */
    public native String stringFromJNI();
}
